use serde::{Deserialize, Serialize};

/// Configuration for connecting to the database.
#[derive(Clone, Serialize, Deserialize, Debug)]
pub struct DatabaseConfig {
    pub name: String,
    #[cfg(feature = "postgres")]
    pub url: String,
    #[cfg(feature = "postgres")]
    pub user: String,
    #[cfg(feature = "postgres")]
    pub password: String,
}

impl Default for DatabaseConfig {
    fn default() -> Self {
        Self {
            name: "atto.db".to_string(),
            #[cfg(feature = "postgres")]
            url: "localhost:5432".to_string(),
            #[cfg(feature = "postgres")]
            user: "postgres".to_string(),
            #[cfg(feature = "postgres")]
            password: "postgres".to_string(),
        }
    }
}

pub trait Configuration {
    fn db_config(&self) -> DatabaseConfig;
}
